#include "CIOboard.h"
#include <string.h>

CIOBoard::CIOBoard(const bool OnlyMonitoring, const unsigned BoardModel)
	: is_connected(false)
	, is_data_ready(false)
	, DIP(0)
	, first_read(true)
	, only_monitoring(OnlyMonitoring)
	, board_model(BoardModel)
	, DI(NULL)
	, AI(NULL)
	, DO(NULL)
	, DO_BACK(NULL)
	, DO_REAL(NULL)
	, PWM(NULL)
	, PWM_BACK(NULL)
{
	switch(BoardModel)
	{
	default:
	case CANLAB_BOARD_MODEL_1:
		NUMBER_OF_DI=BOARD_MODEL_1_DI;
		NUMBER_OF_DO=BOARD_MODEL_1_DO;
		NUMBER_OF_AI=BOARD_MODEL_1_AI;
		NUMBER_OF_PWM = BOARD_MODEL_1_PWM;
		break;
	case VRG_BOARD_MODEL_1:
		NUMBER_OF_DI=BOARD_MODEL_2_DI;
		NUMBER_OF_DO=BOARD_MODEL_2_DO;
		NUMBER_OF_AI=BOARD_MODEL_2_AI;
		NUMBER_OF_PWM = BOARD_MODEL_2_PWM;
		break;
	case VRG_BOARD_MODEL_2:
		NUMBER_OF_DI=BOARD_MODEL_3_DI;
		NUMBER_OF_DO=BOARD_MODEL_3_DO;
		NUMBER_OF_AI=BOARD_MODEL_3_AI;
		NUMBER_OF_PWM = BOARD_MODEL_3_PWM;
		break;
	}

	if(NUMBER_OF_DI)	DI = new unsigned char[NUMBER_OF_DI/8];
	if(NUMBER_OF_AI)	AI = new std::uint16_t[NUMBER_OF_AI];
	if(NUMBER_OF_DO)
	{
		DO = new unsigned char[NUMBER_OF_DO];
		DO_BACK = new unsigned char[NUMBER_OF_DO];
		DO_REAL = new unsigned char[NUMBER_OF_DO/8];
	}
	if(NUMBER_OF_PWM)
	{
		PWM = new std::uint16_t[NUMBER_OF_PWM];
		PWM_BACK = new std::uint16_t[NUMBER_OF_PWM];
	}

	if(DI) memset((void*)&DI[0], 0x00, NUMBER_OF_DI/8);
	if(AI) memset((void*)&AI[0], 0x00, NUMBER_OF_AI*sizeof(std::uint16_t));
	if(DO) memset((void*)&DO[0], 0x01, NUMBER_OF_DO);
	if(DO_BACK) memset((void*)&DO_BACK[0], 0x00, NUMBER_OF_DO);
	if(DO_REAL) memset((void*)&DO_REAL[0], 0x00, NUMBER_OF_DO/8);
	if(PWM) memset((void*)&PWM[0], 0x00, NUMBER_OF_PWM*sizeof(std::uint16_t));
	if(PWM_BACK) memset((void*)&PWM_BACK[0], 0x00, NUMBER_OF_PWM*sizeof(std::uint16_t));
}

CIOBoard::~CIOBoard()
{
	if(DI) delete[] DI;
	if(AI) delete[] AI;
	if(DO) delete[] DO;
	if(DO_BACK) delete[] DO_BACK;
	if(DO_REAL) delete[] DO_REAL;
	if(PWM) delete[] PWM;
	if(PWM_BACK) delete[] PWM_BACK;
}

unsigned int CIOBoard::GetDIP()
{
	std::lock_guard<std::mutex> lock(cs);
	return DIP;
}

bool CIOBoard::GetDIState(const unsigned int InputIndex)
{
	std::lock_guard<std::mutex> lock(cs);
	if(InputIndex>=NUMBER_OF_DI)
		return 0;
#ifdef DI_INVERTED_LOGIC
	if(DI[InputIndex/8]&(0x01<<(InputIndex%8)))
		return false;
	return true;
#else
	if(DI[InputIndex/8]&(0x01<<(InputIndex%8)))
		return true;
	return false;
#endif
}

std::uint64_t CIOBoard::GetDI(void)
{
	std::lock_guard<std::mutex> lock(cs);
#ifdef DI_INVERTED_LOGIC
	return ~((std::uint64_t)DI[0]|((std::uint64_t)DI[1]<<8)|((std::uint64_t)DI[2]<<16)|((std::uint64_t)DI[3]<<24)|((std::uint64_t)DI[4]<<32));
#else
	return ((std::uint64_t)DI[0]|((std::uint64_t)DI[1]<<8)|((std::uint64_t)DI[2]<<16)|((std::uint64_t)DI[3]<<24)|((std::uint64_t)DI[4]<<32));
#endif
}

unsigned int CIOBoard::GetAIState(const unsigned int InputIndex)
{
	std::lock_guard<std::mutex> lock(cs);
	if(InputIndex>=NUMBER_OF_AI)
		return 0;
	return AI[InputIndex];
}

bool CIOBoard::SetDOState(const unsigned int OutputIndex, const DO_STATE state)
{
	std::lock_guard<std::mutex> lock(cs);
	if(OutputIndex<NUMBER_OF_DO)
	{
		DO[OutputIndex] = (unsigned char)state;
		return true;
	}
	return false;
}

bool CIOBoard::SetPWMValue(const unsigned int PWMIndex, const unsigned int value)
{
	std::lock_guard<std::mutex> lock(cs);
	if(PWMIndex<NUMBER_OF_PWM)
	{
		PWM[PWMIndex] = (std::uint16_t)value;
		return true;
	}
	return false;
}

void CIOBoard::SetDOStateByMask(const unsigned char Mask, const DO_STATE state)
{
	std::lock_guard<std::mutex> lock(cs);
	for(unsigned i=0;i<NUMBER_OF_DO;i++)
	{
		if((Mask>>i)&0x01)
		{
			DO[i]=(unsigned char)state;
		}
	}
}

DO_STATE CIOBoard::GetDORequestedState(const unsigned int OutputIndex)
{
	std::lock_guard<std::mutex> lock(cs);
	if(OutputIndex<NUMBER_OF_DO)
	{
		return (DO_STATE)DO[OutputIndex];
	}
	return DO_STATE_UNKNOWN;
}

unsigned int CIOBoard::GetPWMRequestedState(const unsigned int PWMIndex)
{
	std::lock_guard<std::mutex> lock(cs);
	if(PWMIndex<NUMBER_OF_PWM)
	{
		return PWM[PWMIndex];
	}
	return 0;
}

bool CIOBoard::GetDORealState(const unsigned int OutputIndex)
{
	std::lock_guard<std::mutex> lock(cs);
	if(OutputIndex>=NUMBER_OF_DO)
		return 0;
	if(DO_REAL[OutputIndex/8]&(0x01<<(OutputIndex%8)))
		return true;
	return false;
}

//Vraci skutecny stav na PWM vystupu.
unsigned int CIOBoard::GetPWMRealState(const unsigned int PWMIndex)
{
	std::lock_guard<std::mutex> lock(cs);
	if(PWMIndex<NUMBER_OF_PWM)
	{
		return PWM_BACK[PWMIndex];
	}
	return 0;
}

bool CIOBoard::IsConnected(void)
{
	return is_connected;
}

bool CIOBoard::IsDataReady(void)
{
	return is_data_ready;
}
