#ifndef IOBOARD_CARD_H
#define IOBOARD_CARD_H

#include <string>
#include <thread>
#include <mutex>
#include <chrono>
#include <cstdint>
#include "hidapi.h"

//#define	DI_INVERTED_LOGIC

//--------------------------------------------------------------
//BOARD CANLAB - originalni IO deska CANLAB
#define CANLAB_BOARD_MODEL_1		1

#define BOARD_MODEL_1_DI			24
#define BOARD_MODEL_1_DO			16
#define BOARD_MODEL_1_AI			10
#define BOARD_MODEL_1_PWM			2

//--------------------------------------------------------------
//BOARD VRG 1 - zakaznicka deska
#define VRG_BOARD_MODEL_1			2

#define BOARD_MODEL_2_DI			40
#define BOARD_MODEL_2_DO			8
#define BOARD_MODEL_2_AI			10
#define BOARD_MODEL_2_PWM			0

//--------------------------------------------------------------
//BOARD VRG 2 - zakaznicka deska
#define VRG_BOARD_MODEL_2			3

#define BOARD_MODEL_3_DI			40
#define BOARD_MODEL_3_DO			8
#define BOARD_MODEL_3_AI			0
#define BOARD_MODEL_3_PWM			2
//--------------------------------------------------------------

#define DATA_BLOCK_DI				0xA0
#define DATA_BLOCK_DO				0xA1
#define DATA_BLOCK_DO_REAL			0xA2
#define DATA_BLOCK_AI				0xA3
#define DATA_BLOCK_DIP				0xA4
#define DATA_BLOCK_PWM				0xA5
#define DATA_BLOCK_MAGIC			0xAF

typedef enum {
	DO_STATE_UNKNOWN = 0,
	DO_STATE_OFF,
	DO_STATE_ON,
	DO_STATE_BLINK_100MS = 50,
	DO_STATE_BLINK_250MS,
	DO_STATE_BLINK_500MS,
	DO_STATE_BLINK_1000MS,
	DO_STATE_BLINK_1500MS,
	DO_STATE_BLINK_2000MS,
	DO_STATE_BLINK_ONCE_100MS = 100,
	DO_STATE_BLINK_ONCE_250MS,
	DO_STATE_BLINK_ONCE_500MS,
	DO_STATE_BLINK_ONCE_1000MS,
	DO_STATE_BLINK_ONCE_1500MS,
	DO_STATE_BLINK_ONCE_2000MS,
	DO_STATE_BLINK_ONCE_5000MS,
	DO_STATE_BLINK_ONCE_10000MS,
	DO_STATE_UNCHANGED=254,
	DO_STATE_BLINK_ONCE_WAIT=255,
} DO_STATE;

typedef enum {
	IO_BOARD_CON_USB = 0,
	IO_BOARD_CON_TCP
}E_IO_BOARD_CONNECTION_TYPE;

class CIOBoard
{
public:
	CIOBoard(const bool OnlyMonitoring, const unsigned BoardModel);
	virtual ~CIOBoard();
	
	//Vraci hodnotu konfiguracniho DIP prepinace
	//DIP1-5	- cislo zarizeni v USB identifikatoru, volitelne se muze pricitat k nastavene IP adrese
	//DIP6		- aktivace bootloaderu
	//DIP7		- je li ON, je hodnota na DIP1-5 pricitana k IP adrese
	//DIP8		- defaultni nastaveni, IP:192.168.110.77:5000
	unsigned int GetDIP();

	//Vraci stav digitalniho vstupu. Vstupy se spinaji prizemnenim.
	//Logiku lze zmenit definovanim DI_INVERTED_LOGIC
	bool GetDIState(const unsigned int InputIndex);

	//Vraci hodnotu vsech 40 dig. vstupu jako dolnich 40 bitu vracene hodnoty.
	std::uint64_t GetDI(void);

	//Vraci hodnotu analogoveho vystupu, rozsah hodnot je 0..10000.
	unsigned int GetAIState(const unsigned int InputIndex);
	double GetAIState(const unsigned int InputIndex, double *X, double *Y, int Dim);

	//Dig. vystupy jsou nastaveny podle bitu v masce ktere jsou nastaveny do 1, na vystup je nastaven stav specifikovaneho hodnotou State (ON/OFF/BLINKx).
	void SetDOStateByMask(const unsigned char Mask, const DO_STATE State);

	//Nastavuje vybrany digitalni vystup na zadany stav (ON/OFF/BLINKx).
	bool SetDOState(const unsigned int OutputIndex, const DO_STATE State);

	//Nastavuje hodnotu PWM vystupu
	bool SetPWMValue(const unsigned int PWMIndex, const unsigned int Value);
	bool SetPWMValue(const unsigned int PWMIndex, const double value, double *X, double *Y, int Dim);

	//Vraci pozadovany stav na vystupu.
	DO_STATE GetDORequestedState(const unsigned int OutputIndex);

	//Vraci pozadovany stav na vystupu.
	unsigned int GetPWMRequestedState(const unsigned int PWMIndex);

	//Vraci skutecny stav na dig. vystupu - jedna se o realny stav podle poriody prikazu blink.
	bool GetDORealState(const unsigned int OutputIndex);

	//Vraci skutecny stav na PWM vystupu.
	unsigned int GetPWMRealState(const unsigned int PWMIndex);

	//true pokud je io_board pripojen
	bool IsConnected(void);

	//true pokud jsou prectena data zpet z desky - data jsou po pripojeni validni
	bool IsDataReady(void);

	double LinearInterpolation(double *X, double *Y, int Dim, double dX);

	//Typ pripojeni (USB/TCP)
	virtual E_IO_BOARD_CONNECTION_TYPE GetConnectionType(void) = 0;

	virtual bool Timer(void) = 0;

	//Pocty vstupu a vystupu IO boardu
	unsigned GetNumberOfDI() {return NUMBER_OF_DI;};
	unsigned GetNumberOfDO() {return NUMBER_OF_DO;};
	unsigned GetNumberOfAI() {return NUMBER_OF_AI;};
	unsigned GetNumberOfPWM() {return NUMBER_OF_PWM;};

protected:
	bool is_connected;
	bool is_data_ready;
	bool first_read;
	bool only_monitoring;
	std::uint32_t board_model;

	unsigned NUMBER_OF_DI;
	unsigned NUMBER_OF_AI;
	unsigned NUMBER_OF_DO;
	unsigned NUMBER_OF_PWM;

	unsigned char *DI;
	std::uint16_t *AI;
	std::uint16_t *PWM;
	std::uint16_t *PWM_BACK;
	unsigned char *DO;
	unsigned char *DO_BACK;
	unsigned char *DO_REAL;

	unsigned char DIP;

	std::mutex cs;
};

#endif