#include "CIOBoardTCPBootloader.h"
#include <iostream>
#include <fstream>

CIOBoardTCPBootloader::CIOBoardTCPBootloader(const std::string Address, const unsigned Port, std::string HexFilePath, const bool CreateOwnThread)
	: address(Address)
	, port(Port)
	, stop_thread(false)
	, own_thread(CreateOwnThread)
	, is_connected(false)
	, next_hex_row(-1)
	, status(BOOT_STATUS_WAIT_FOR_CONNECTION)
{
	if(LoadHexFile(HexFilePath))
	{
		if(own_thread)
			timer_thread = std::thread(&CIOBoardTCPBootloader::TimerThread,this);
	}
}

CIOBoardTCPBootloader::~CIOBoardTCPBootloader()
{
	stop_thread = true;
	SetNonBlocking(true);
	if(own_thread)
	{
		if(timer_thread.joinable()) 
			timer_thread.join();
	}
	if(is_connected)
	{
	#ifdef _LINUX
        close(socket_id);
	#else
        closesocket(socket_id);
    #endif
	}
}
 
bool CIOBoardTCPBootloader::LoadHexFile(std::string filepath)
{
	std::ifstream ifile;
	char buffer[512];
	ifile.open(filepath);
	
	if (!ifile.is_open())
	{
		status = BOOT_STATUS_BAD_FILE;
		return false;
	}
	while(ifile.getline(buffer,511,'\n'))
	{
		if(buffer[0]==':')
		{
			hex.push_back(std::string(&buffer[1]));
		}
		if (ifile.eof())
			break;
	}
	status = BOOT_STATUS_WAIT_FOR_CONNECTION;
	return true;
}

bool CIOBoardTCPBootloader::Connect(void)
{
	WORD wVersionRequested = MAKEWORD(1,1);
	WSADATA data;
	int error_code=0;
	if (WSAStartup(wVersionRequested, &data) != 0)
	{
		std::cerr << "IOboard error: WSAStartup" << std::endl;
		GetLastError();
	}
    TIMEVAL Timeout;
    Timeout.tv_sec = 2;
    Timeout.tv_usec = 0;
    struct sockaddr_in serverAddress; 
 
    socket_id = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
 
    serverAddress.sin_addr.s_addr = inet_addr(address.c_str());
    serverAddress.sin_port = htons(port); 
    serverAddress.sin_family = AF_INET;
 
    //non-blocking
    unsigned long iMode = 1;
    int iResult = ioctlsocket(socket_id, FIONBIO, &iMode);
    if (iResult != NO_ERROR)
    {
		is_connected=false;
		return false;
    }
	    
    if(connect(socket_id,(struct sockaddr *)&serverAddress,sizeof(serverAddress))==false)
    {	
		int e = WSAGetLastError();
		std::cerr << "IOboard error: connect, " << e << std::endl;
		is_connected=false;
        return false;
    }	
 
    iMode = 0;
    iResult = ioctlsocket(socket_id, FIONBIO, &iMode);
    if (iResult != NO_ERROR)
    {	
		is_connected=false;
	#ifdef _LINUX
        close(socket_id);
	#else
        closesocket(socket_id);
    #endif
		return false;
    }
 
    fd_set Write, Err;
    FD_ZERO(&Write);
    FD_ZERO(&Err);
    FD_SET(socket_id, &Write);
    FD_SET(socket_id, &Err);
 
    //is socket ready
    select(0,NULL,&Write,&Err,&Timeout);			
    if(FD_ISSET(socket_id, &Write)) 
    {	
		is_connected=true;
        return true;
    }
 
	is_connected=false;
	#ifdef _LINUX
        close(socket_id);
	#else
        closesocket(socket_id);
    #endif
    return false;
}

void CIOBoardTCPBootloader::SetNonBlocking(bool non_blocking)
{
	unsigned long mode = non_blocking ? 1 : 0;
	ioctlsocket(socket_id, FIONBIO, &mode);
}

bool CIOBoardTCPBootloader::Timer(void)
{	
	if(is_connected==false)
	{
		bool ret = Connect();
		if(ret)
			status=BOOT_STATUS_ERASE;
		return ret; 
	}
    else if((status==BOOT_STATUS_ERASE) || (status==BOOT_STATUS_RUNNING))
	{
		if(!SendRow())
		{
		#ifdef _LINUX
			close(socket_id);
		#else
			closesocket(socket_id);
		#endif
			is_connected=false;
			return false;
		}
	
		int num=0;
		unsigned char buffer[128];	
		memset(buffer,0,128);
		IO_BOARD_PACKET_HEX_ROW_RESPONSE packet;

		num = recv(socket_id,(char*)&packet.header,sizeof(IO_BOARD_PACKET_HEADER),0);
		if(num==sizeof(IO_BOARD_PACKET_HEADER) && packet.header.magic==PACKET_MAGIC)
		{
			if(packet.header.length > 0 && packet.header.packet_type == PACKET_BOOT)
			{
				num = recv(socket_id,(char*)&packet.hex_row_index,packet.header.length,0);
				if (num == -1)
				{
				#ifdef _LINUX
					close(socket_id);
				#else
					closesocket(socket_id);
				#endif
					is_connected=false;
					return false;
				}
				if(packet.hex_row_index==0xFFFFFFFF)
				{
					next_hex_row=0;
				}
				else if(packet.hex_row_index==(hex.size()-1))
				{
					status = BOOT_STATUS_DONE;
				}
				else
				{
					next_hex_row++;
				}
			}
		}
		if (num == -1)
		{
		#ifdef _LINUX
			close(socket_id);
		#else
			closesocket(socket_id);
		#endif
			is_connected=false;
			return false;
		}
	}
	return is_connected;
}

bool CIOBoardTCPBootloader::SendRow(void)
{
	IO_BOARD_PACKET_HEX_ROW packet; 
	packet.header.magic = 0x11223456;
	packet.header.packet_type = PACKET_BOOT;
	packet.header.length=HEX_ROW_SIZE+4;
	memset(&packet.hex_row[0],0,HEX_ROW_SIZE);
	if(next_hex_row==-1)
	{
		status=BOOT_STATUS_ERASE;
		packet.hex_row_index = 0xFFFFFFFF;
	}
	else
	{
		status=BOOT_STATUS_RUNNING;
		packet.hex_row_index = next_hex_row;
		memcpy(&packet.hex_row[0],hex[next_hex_row].c_str(),hex[next_hex_row].length());
	}		
	int num= send(socket_id,(char*)&packet,sizeof(IO_BOARD_PACKET_HEADER)+packet.header.length,0);
	if (num == -1)
	{
		is_connected=false;
		return false;
	}
	return true;
}

void CIOBoardTCPBootloader::TimerThread()
{
    while(!stop_thread)
	{
		if(Timer()==false)
			std::this_thread::sleep_for( std::chrono::microseconds(250) );
		else
			std::this_thread::sleep_for( std::chrono::microseconds(20) );
    }
}



